<?php
	//--------------------------------------------------------------------------------
	//	Php FeedWriter 3.2 - Sample Usage
	//
	//	(c) Copyright Daniel Soutter
	//	
	//	Website: http://phpFeedWriter.WebmasterHub.net/
	//	
	//  Php FeedWriter and this sample script is provided with no warranty and may be 
	//  used or developed at your own risk, providing that you have read and aggree to
	//	the Terms of Use http://phpfeedwriter.webmasterhub.net/terms .
	//
	//	Please post any comments, bugs or suggestions for improvement to the website.
	//
	//	For usage instructions or technical information about Php FeedWriter, see the 
	//  online documentation: http://phpFeedWriter.WebmasterHub.net/docs
	//
	//  The feed and database configuration details for this example are set in 
	//  the separate feed-config.php file provided as part of this sample.
	//
	//  For information about this script, please see:
	//	http://phpfeedwriter.webmasterhub.net/2012/01/db-feed-sample-php-explained-sample-feed-database-integration-php-script/
	//--------------------------------------------------------------------------------
	
	// Include the config file containing variables for feed channel data and the database connection / object.
	include('feed-config.php');
	
	//Include FeedWriter class file
	//--------------------------------
	require_once("feedw/FeedWriter.php");
		
	//1. Instantiate new FeedWriter object, passing the title, description and link
	//--------------------------------
	$feed = new FeedWriter(
		$feed_title, 	
		$feed_description, 
		$feed_url, 
		6, //indent
		true, //Use CDATA
		null, //encoding
		true //enable validation
		);
		
	//Enable debug
	$feed->debug = true;
	
	//Connect to the database
	if (!$dbConnection = mysql_connect($feed_db_host . ':' . $feed_db_port, $feed_db_user, $feed_db_pass)) 
	{
		die('Error Connecting To Database');
	}
	else //Connected successfully
	{
		//	Select the database
		if (!mysql_select_db($feed_db_dbname, $dbConnection)) {
			die('Error selecting database');
		}
	}
	
	//Get the format to display in, 
	//set to RSS 2.0 as default or if format is not found
	$format = RSS_2_0;
	if(isset($_GET['format']) && $_GET['format'] != null)
	{
		foreach($feed->getFeedFormats() as $curFormat)
		{
			if($curFormat[0] == $_GET['format'])
				$format = $curFormat[1];
		}
	}
	
	//Build the SELECT query if not manually set in the config file.
	//----------------------------------
	//If $select_query has been manually set in theconfig file to a custom select query, the custom query will be used.
	//If $select_query is set to NULL in the config file, the query is generated dynamically.
	if($select_query == null)
	{
		$colsIncluded = 0;
		$select_query = "SELECT ";

		//Loop through table columns and add to query if not null
		foreach($feed_item_table_cols as $curCol){
			if($colsIncluded == 0 && $curCol != null){
				//First
				$select_query .= $curCol;
				$colsIncluded++;
			}
			elseif($curCol != null){
				$select_query .= ", $curCol";
				$colsIncluded++;
			}
		}
	}
	
	//Append remain components of query
	$select_query .= " FROM $feed_items_table ";
	//Order By
	$select_query .= "ORDER BY " . $feed_item_table_cols['item_date_column'] . " DESC ";
	//Row Limit
	if($feed_item_limit != null && $feed_item_limit > 0)
		$select_query .= " LIMIT 0, $feed_item_limit ";
	//Finish query
	$select_query .= "; ";
	 
	//echo $select_query;
	
	
	//Retrieve the records from the database to be added as items to the feed.
	//Check that the DB Connection variable has been instantiated (by the config file - feed-config.php)
	if(isset($dbConnection) && $dbConnection != false){
		if(!($mysql_result = mysql_query($select_query, $dbConnection))){
			//Display the database error.
			if($feed->debug)
				die("<b>Error " . mysql_errno() . "</b><br/>" . mysql_error());
			else
				exit;
		}
		//Store the associative record set in an array.
		while(($feedItems[] = mysql_fetch_assoc($mysql_result)) || array_pop($feedItems)); 
	}
	else {
		die("Connection unavailable...");
	}	
	
	//End of Feed Configuration --- 
	
	
	//Start populating the channel data and feed items with data:
	//---------------------------------
	// The variables containing channel data are set in the feed-config.php file which comes as part of 
	// this sample script.
	
	//Add channel data to the feed
	$feed->set_image($channel_image['imageTitle'], $channel_image['imageSrc'], $channel_image['imageUrl']);
	$feed->set_selfLink($feed_selfLink); 
	$feed->set_author($feed_author['email'], $feed_author['name'],$feed_author['url']);
	$feed->set_date($date_feed_updated,DATE_UPDATED);
	$feed->set_language($feed_Language);
	
	$feed->set_webmaster($feed_webmaster);
	$feed->set_skipDays($skip_days);
	$feed->set_skipHours($skip_hours);
	$feed->set_input($feed_input['input_title'],$feed_input['input_desc'],$feed_input['input_name'],$feed_input['input_url']); 
	$feed->set_cloud($feed_cloud['cloud_domain'],$feed_cloud['cloud_port'],$feed_cloud['cloud_path'],$feed_cloud['cloud_reg_procedure'], $feed_cloud['cloud_protocol']); 
	$feed->set_date($date_feed_published,DATE_PUBLISHED);
	$feed->set_id($feed_id);
	$feed->set_copyright($feed_copyright_notice);
	$feed->add_contributor($feed_contributor['email'], $feed_contributor['name'],$feed_contributor['url']);
		
	//Add categories to the channel
	foreach($feedCategories as $curCategory) {
		$feed->add_category($curCategory);
	}
	
	//Add items to the feed
	//--------------------------------
	if(count($feedItems) > 0)
	{
		foreach($feedItems as $curItem)
		{
			//obtain data availble for the current feed item.  
			//----------------------------
			//The example has been configured this way to ease the process of 
			//adjusting the database column mappings to feed item elements, and to make the example more generic.  
			//As a result, this process could be similified if required to suit more specific needs.
			
			if($feed_item_table_cols['item_id_column'] != null)
				$item_id = $curItem[$feed_item_table_cols['item_id_column']];
			else
				$item_id = null;	
				
			if($feed_item_table_cols['item_title_column'] != null)
				$item_title = $curItem[$feed_item_table_cols['item_title_column']];
			else
				$item_title = 'Untitled';
				
			if($feed_item_table_cols['item_content_column'] != null)
				$item_body = $curItem[$feed_item_table_cols['item_content_column']];
			else
				$item_body = null;
				
			if($feed_item_table_cols['item_date_column'] != null)
				$date_modified = $curItem[$feed_item_table_cols['item_date_column']];
			else
				$date_modified = null;
				
			if($feed_item_table_cols['item_link_column'] != null)
				$item_link = $curItem[$feed_item_table_cols['item_link_column']];
			elseif($item_link_base_url != null) {
				$item_link = $item_link_base_url . $item_id;
			}
			else
				$item_link = null;
				
			if($feed_item_table_cols['item_copyright_column'] != null)
				$item_copyright = $curItem[$feed_item_table_cols['item_copyright_column']];
			else
				$item_copyright = null;
				
			if($feed_item_table_cols['item_author_name_column'] != null)
				$item_author_name = $curItem[$feed_item_table_cols['item_author_name_column']];
			else
				$item_author_name = null;
			
			//Example assumes a single category is stored in each feed item record.
			if($feed_item_table_cols['item_category_column'] != null)
				$category = $curItem[$feed_item_table_cols['item_category_column']];
			else
				$category = null;
		
			//Add the current item to the feed
			$feed->add_item($item_title, $item_body, $item_link);
			
			//Add the feed item modified date if available
			if($date_modified != null)
				$feed->set_date($date_modified, DATE_UPDATED);
			
			//Set the ID of the current feed item to the link if not null
			if($item_link != null)
				$feed->set_id($item_link);
			
			//Add copyright details if available
			if($item_copyright != null)
				$feed->set_copyright($item_copyright);
			
			//Add the author name if supplied - set_author(email, name, uri)
			if($item_author_name != null)
				$feed->set_author(null, $item_author_name, null);
			
			//Set the comments url to the link for the feed item (common, but optional)
			if($item_link != null)
				$feed->set_comments($item_link);
			
			//Add the category to the item.  This function should be called multiple times to add more than one category to a single feed item.
			$feed->add_category($category);
		}
	}
	
	//Set the item body/content output type to html for the selected output format
	$feed->set_feedConstruct($format);
	$feed->feed_construct->construct['itemContent']['type'] = 'html';
	
	//Build and output the feed XML in the specified, or default format
	//--------------------------------
	echo $feed->getXML($format);
?>